from typing import overload
from enum import IntEnum
import System
import System.Security.Authentication


class SslProtocols(IntEnum):
    """This class has no documentation."""

    NONE = 0

    SSL_2 = ...

    SSL_3 = ...

    TLS = ...

    TLS_11 = ...

    TLS_12 = ...

    TLS_13 = ...

    DEFAULT = ...


class ExchangeAlgorithmType(IntEnum):
    """This class has no documentation."""

    NONE = 0

    RSA_SIGN = ...

    RSA_KEY_X = ...

    DIFFIE_HELLMAN = ...


class CipherAlgorithmType(IntEnum):
    """This class has no documentation."""

    NONE = 0

    RC_2 = ...

    RC_4 = ...

    DES = ...

    TRIPLE_DES = ...

    AES = ...

    AES_128 = ...

    AES_192 = ...

    AES_256 = ...

    NULL = ...


class HashAlgorithmType(IntEnum):
    """This class has no documentation."""

    NONE = 0

    MD_5 = ...

    SHA_1 = ...

    SHA_256 = ...

    SHA_384 = ...

    SHA_512 = ...


